@@ -478,7 +478,7 @@ get.quadrant.f <- function(start.x, start.y, end.x, end.y, tol = 0.0000001) {
   x.diff <- (start.x - end.x)
   y.diff <- (start.y - end.y)
 
-  if (abs(x.diff) < tol & abs(y.diff) < tol) print("Segments on top of each other!")
+  if (abs(x.diff) < tol && abs(y.diff) < tol) print("Segments on top of each other!")
 
   quad <- NA
 
@@ -495,13 +495,13 @@ get.quadrant.f <- function(start.x, start.y, end.x, end.y, tol = 0.0000001) {
     if (end.x < start.x) quad <- 4
   }
   # NE
-  if (end.x > start.x & end.y > start.y) quad <- 5
+  if (end.x > start.x && end.y > start.y) quad <- 5
   # SE
-  if (end.x > start.x & end.y < start.y) quad <- 6
+  if (end.x > start.x && end.y < start.y) quad <- 6
   # SW
-  if (end.x < start.x & end.y < start.y) quad <- 7
+  if (end.x < start.x && end.y < start.y) quad <- 7
   # NW
-  if (end.x < start.x & end.y > start.y) quad <- 8
+  if (end.x < start.x && end.y > start.y) quad <- 8
 
   if (is.na(quad)) print("Quadrant not assigned")
 
@@ -531,7 +531,7 @@ geo.distance.f <- function(lon1, lat1, lon2, lat2) {
   rad <- pi / 180
   nm2km <- 1.852
 
-  if ((lat1 == lat2) & (lon1 == lon2)) {
+  if ((lat1 == lat2) && (lon1 == lon2)) {
     posdist <- 0
   } else {
     rlat1 <- lat1 * rad
@@ -718,12 +718,12 @@ get.coords.f <- function(quad = NULL, alpha = NULL, new.x = NULL, new.y = NULL,
   deg2rad <- pi / 180
 
   # Quadrants 1 and 3 (i.e. heading directly N or S, respectively)
-  if (quad == 1 | quad == 3) {
+  if (quad == 1 || quad == 3) {
     sgt.x <- new.x + (side * pd)
     sgt.y <- new.y
   }
   # Quadrants 2 and 4 (i.e. heading directly E or W, respectively)
-  if (quad == 2 | quad == 4) {
+  if (quad == 2 || quad == 4) {
     sgt.x <- new.x
     sgt.y <- new.y + (side * pd)
   }
