@@ -1,14 +1,14 @@
 # GENERICS ----
 
-#' Construct A-matrix
+#' Construct component linearisations
 #'
-#' Constructs the A-matrix for components and data
+#' Constructs the linearisation mapper for each component
 #' @export
-#' @rdname amatrix_eval
-amatrix_eval <- function(...) {
-  UseMethod("amatrix_eval")
+#' @rdname comp_lin_eval
+comp_lin_eval <- function(...) {
+  UseMethod("comp_lin_eval")
 }
-#' Obtain covariate values
+#' Obtain component inputs
 #'
 #' @export
 #' @rdname input_eval
@@ -162,7 +162,8 @@ component <- function(...) {
 #' should be evaluated (coordinates, indices, continuous scalar (for rw2 etc)).
 #' Arguments starting with weights, group, replicate behave similarly to main,
 #' but for the corresponding features of `INLA::f()`.
-#' @param model Either one of "offset", "factor_full", "factor_contrast", "linear",
+#' @param model Either one of "const" (same as "offset"), "factor_full",
+#' "factor_contrast", "linear",
 #' "fixed", or a model name or
 #' object accepted by INLA's `f` function. If set to NULL, then "linear" is used
 #' for vector inputs, and "fixed" for matrix input (converted internally to
@@ -173,14 +174,16 @@ component <- function(...) {
 #' extracts the mesh object to use as the mapper, and auto-generates mappers
 #' for indexed models. (Default: NULL, for auto-determination)
 #' @param main_layer,main_selector
-#' The `_layer` is a numeric index or character name of which
-#' layer/variable to extract from a covariate data object given in `main`
-#' (Default: The effect component name, if it exists i the covariate object,
-#' otherwise the first column of the covariate data frame)
+#' The `_layer` input should evaluate to a numeric index or character name or
+#' vector of which
+#' layer/variable to extract from a covariate data object given in `main`.
+#' (Default: NULL if `_selector` is given. Otherwise the effect component name,
+#'  if it exists in the covariate object, and otherwise the first column of
+#'  the covariate data frame)
 #'
-#' The `_selector` is character name of a variable whose contents determines which layer to
-#' extract from a covariate for each data point. Overrides the `layer`.
-#' (Default: NULL)
+#' The `_selector` value should be a character name of a variable
+#' whose contents determines which layer to extract from a covariate for each
+#' data point. (Default: NULL)
 #' @param n The number of latent variables in the model. Should be auto-detected
 #' for most or all models (Default: NULL, for auto-detection).
 #' An error is given if it can't figure it out by itself.
@@ -210,7 +213,6 @@ component <- function(...) {
 #' replication model. Same syntax as for `main`
 #' @param A.msk TODO: check/fix/deprecate this parameter.
 #' Likely doesn't work at the moment, and I've found no examples that use it.
-# Deprecated parameters
 #' @param .envir Evaluation environment
 #' @param envir_extra TODO: check/fix this parameter.
 #'
@@ -288,7 +290,7 @@ component.character <- function(object,
   #            clinear, sigm, revsigm, log1exp, logdist)
   #
   # Supported:
-  # btypes = c("offset", "factor_full", "factor_contrast", "linear", "clinear", "iid", "seasonal", "rw1", "rw2", "ar", "ar1", "ou", "spde")
+  # btypes = c("const", "offset", "factor_full", "factor_contrast", "linear", "clinear", "iid", "seasonal", "rw1", "rw2", "ar", "ar1", "ou", "spde")
 
   # The label
   label <- object
@@ -311,11 +313,6 @@ component.character <- function(object,
     group_model <- "exchangeable"
   }
 
-  if (is.null(main_layer)) {
-    main_layer <- label
-  }
-
-
   if ("map" %in% names(sys.call())) {
     #    if (!is.null(substitute(map))) {
     if (is.null(substitute(main))) {
@@ -367,7 +364,7 @@ component.character <- function(object,
       input = bru_input(
         substitute(main),
         label = label,
-        layer = main_layer,
+        layer = substitute(main_layer),
         selector = main_selector
       ),
       mapper = mapper,
@@ -380,7 +377,7 @@ component.character <- function(object,
       input = bru_input(
         substitute(group),
         label = paste0(label, ".group"),
-        layer = group_layer,
+        layer = substitute(group_layer),
         selector = group_selector
       ),
       mapper = group_mapper,
@@ -391,7 +388,7 @@ component.character <- function(object,
       input = bru_input(
         substitute(replicate),
         label = paste0(label, ".repl"),
-        layer = replicate_layer,
+        layer = substitute(replicate_layer),
         selector = replicate_selector
       ),
       mapper = replicate_mapper,
@@ -405,7 +402,7 @@ component.character <- function(object,
         bru_input(
           substitute(weights),
           label = paste0(label, ".weights"),
-          layer = weights_layer,
+          layer = substitute(weights_layer),
           selector = weights_selector
         )
       },
@@ -444,23 +441,25 @@ component.character <- function(object,
   }
 
   # Special and general cases:
-  if (component$main$type %in% c("offset")) {
+  if (component$main$type %in% c("offset", "const")) {
     # The offset is included either automatically for ~ . linear models,
     # or explicitly by name in the predictor expression, so no INLA formula
     # component is needed.
     component$inla.formula <- as.formula(paste0("~ ."),
       env = .envir
     )
-    component$main$mapper <- bru_mapper_offset()
+    component$main$mapper <- bru_mapper_const()
     component$group$mapper <- bru_mapper_index(1L)
     component$replicate$mapper <- bru_mapper_index(1L)
-    # Add multi-mapper
+    # Add scalable multi-mapper
     component[["mapper"]] <-
-      bru_mapper_multi(list(
-        main = component$main$mapper,
-        group = component$group$mapper,
-        replicate = component$replicate$mapper
-      ))
+      bru_mapper_scale(
+        bru_mapper_multi(list(
+          main = component$main$mapper,
+          group = component$group$mapper,
+          replicate = component$replicate$mapper
+        ))
+      )
   } else {
     if (!is.null(copy)) {
       # Store copy-model name or object in the environment
@@ -674,7 +673,7 @@ component_list.list <- function(object,
   stopifnot(all(vapply(object, function(x) inherits(x, "component"), TRUE)))
   names(object) <- lapply(object, function(x) x$label)
   if (anyDuplicated(names(object))) {
-    warning(paste0(
+    stop(paste0(
       "Duplicated component labels detected: ",
       paste0(
         "'",
@@ -774,13 +773,15 @@ add_mappers.component <- function(component, lhoods, ...) {
     env = component$env,
     require_indexed = TRUE
   )
-  # Add multi-mapper
+  # Add scalable multi-mapper
   component[["mapper"]] <-
-    bru_mapper_multi(list(
-      main = component$main$mapper,
-      group = component$group$mapper,
-      replicate = component$replicate$mapper
-    ))
+    bru_mapper_scale(
+      bru_mapper_multi(list(
+        main = component$main$mapper,
+        group = component$group$mapper,
+        replicate = component$replicate$mapper
+      ))
+    )
 
   fcall <- component$fcall
 
@@ -804,7 +805,7 @@ add_mappers.component <- function(component, lhoods, ...) {
     assign(values_name, component$main$values, envir = component$env_extra)
   }
 
-  if (!identical(component[["main"]][["type"]], "offset")) {
+  if (!(component[["main"]][["type"]] %in% c("offset", "const"))) {
     # Update the formula that will be presented to INLA
     component$inla.formula <-
       as.formula(paste0(
@@ -901,6 +902,9 @@ bru_subcomponent <- function(input = NULL,
     } else if (identical(model, "fixed")) {
       model <- "iid"
       type <- "fixed"
+    } else if (model %in% c("offset", "const")) {
+      model <- "const"
+      type <- "const"
     } else {
       type <- model
     }
@@ -1166,6 +1170,100 @@ make_submapper <- function(subcomp_n,
 }
 
 
+#' Extract mapper information from INLA model component objects
+#'
+#' The component definitions will automatically attempt to extract mapper
+#' information from any model object by calling the generic `bru_get_mapper`.
+#' Any class method implementation should return a [bru_mapper] object suitable
+#' for the given latent model.
+#'
+#' @param model A model component object
+#' @param \dots Arguments passed on to other methods
+#' @return A [bru_mapper] object defined by the model component
+#' @seealso [bru_mapper] for mapper constructor methods, and
+#' [bru_mapper_methods] for method generics and specific implementations.
+#' @export
+#' @examples
+#' if (bru_safe_inla(quietly = TRUE)) {
+#'   library(INLA)
+#'   mesh <- inla.mesh.create(globe = 2)
+#'   spde <- inla.spde2.pcmatern(mesh,
+#'     prior.range = c(1, 0.5),
+#'     prior.sigma = c(1, 0.5)
+#'   )
+#'   mapper <- bru_get_mapper(spde)
+#'   ibm_n(mapper)
+#' }
+bru_get_mapper <- function(model, ...) {
+  UseMethod("bru_get_mapper", model)
+}
+
+
+#' @rdname bru_get_mapper
+#' @details * `bru_get_mapper.inla.spde` extract an indexed mapper for
+#' the `model$mesh` object contained in the model object.
+#' It returns `NULL` gives a warning
+#' if no known mesh type is found in the model object.
+#' @export
+bru_get_mapper.inla.spde <- function(model, ...) {
+  if (inherits(model$mesh, "inla.mesh")) {
+    mapper <- bru_mapper(model$mesh)
+  } else if (inherits(model$mesh, "inla.mesh.1d")) {
+    mapper <- bru_mapper(model$mesh, indexed = TRUE)
+  } else {
+    mapper <- NULL
+    warning(
+      paste0(
+        "Unknown SPDE mesh class '",
+        paste0(class(model$mesh), collapse = ", "),
+        "' for bru_get_mapper.inla.spde. Please specify a mapper manually instead."
+      ),
+      immediate. = TRUE
+    )
+  }
+  mapper
+}
+
+#' @rdname bru_get_mapper
+#' @details * `bru_get_mapper.inla.rgeneric` returns the mapper given by a call to
+#' `model$f$rgeneric$definition("mapper")`. To support this for your own
+#' `inla.rgeneric` models, add a `"mapper"` option to the `cmd` argument
+#' of your rgeneric definition function. You will need to store the mapper
+#' in your object as well.  Alternative, define your model using a subclass
+#' and define a corresponding `bru_get_mapper.subclass` method that should return
+#' the corresponding `bru_mapper` object.
+#' @export
+bru_get_mapper.inla.rgeneric <- function(model, ...) {
+  if (is.null(model[["f"]][["rgeneric"]][["definition"]])) {
+    NULL
+  } else {
+    model[["f"]][["rgeneric"]][["definition"]]("mapper")
+  }
+}
+
+#' @rdname bru_get_mapper
+#' @details * `bru_get_mapper_safely` tries to call the `bru_get_mapper`,
+#' and returns `NULL` if it fails (e.g. due to no available class method).
+#' If the call succeeds and returns non-`NULL`, it checks that the object
+#' inherits from the `bru_mapper` class, and gives an error if it does not.
+#' @export
+bru_get_mapper_safely <- function(model, ...) {
+  m <- tryCatch(
+    bru_get_mapper(model, ...),
+    error = function(e) {
+    }
+  )
+  if (!is.null(m) && !inherits(m, "bru_mapper")) {
+    stop(paste0(
+      "The bru_get_mapper method for model class '",
+      paste0(class(model), collapse = ", "),
+      "' did not return a bru_mapper object"
+    ))
+  }
+  m
+}
+
+
 # Defines a default mapper given the type of model and parameters provided
 # Checks subcomp$mapper, subcomp$model (for "bym2" and other multicomponent
 # models), subcomp$model$mesh (for spde models), subcomp$n,
@@ -1176,19 +1274,8 @@ make_mapper <- function(subcomp,
                         strict = TRUE,
                         require_indexed = FALSE) {
   if (is.null(subcomp[["mapper"]])) {
-    if (subcomp[["type"]] %in% c("spde")) {
-      if (inherits(subcomp[["model"]]$mesh, "inla.mesh")) {
-        subcomp[["mapper"]] <- bru_mapper(subcomp[["model"]]$mesh)
-      } else if (inherits(subcomp[["model"]]$mesh, "inla.mesh.1d")) {
-        subcomp[["mapper"]] <-
-          bru_mapper(subcomp[["model"]]$mesh, indexed = TRUE)
-      } else {
-        stop(paste0(
-          "Unknown SPDE mesh class '",
-          paste0(class(subcomp[["model"]]$mesh), collapse = ", "),
-          "' for ", label, ". Please specify a mapper manually instead."
-        ))
-      }
+    if (!inherits(subcomp[["model"]], "character")) {
+      subcomp[["mapper"]] <- bru_get_mapper_safely(subcomp[["model"]])
     }
   }
   if (!is.null(subcomp[["mapper"]])) {
@@ -1199,11 +1286,10 @@ make_mapper <- function(subcomp,
         "' for ", label
       ))
     }
-    subcomp[["mapper"]] <- bru_mapper(subcomp[["mapper"]])
   } else if (subcomp[["type"]] %in% c("linear", "clinear")) {
     subcomp[["mapper"]] <- bru_mapper_linear()
-  } else if (subcomp[["type"]] %in% c("offset")) {
-    subcomp[["mapper"]] <- bru_mapper_offset()
+  } else if (subcomp[["type"]] %in% c("offset", "const")) {
+    subcomp[["mapper"]] <- bru_mapper_const()
   } else if (subcomp[["type"]] %in% c("fixed")) {
     if (!is.null(subcomp[["values"]])) {
       labels <- subcomp[["values"]]
@@ -1294,7 +1380,7 @@ code.components <- function(components, add = "") {
     if (ix > 0) {
       label <- substr(code, 1, ix - 1)
       is.fixed <- FALSE
-      if (label == "offset") {
+      if (label %in% c("offset", "const")) {
         is.offset <- TRUE
       }
     } else {
@@ -1325,7 +1411,7 @@ code.components <- function(components, add = "") {
             paste0(label, "("),
             paste0(
               fname, '("', label, '"',
-              ', model = "offset", main = '
+              ', model = "const", main = '
             ),
             code,
             fixed = TRUE
@@ -1343,6 +1429,9 @@ code.components <- function(components, add = "") {
   codes
 }
 
+
+
+
 # OPERATORS ----
 
 
@@ -1438,34 +1527,41 @@ print.summary_component_list <- function(x, ...) {
 #' @export
 #' @keywords internal
 #' @param component A component.
-#' @param data A `data.frame` or Spatial* object of covariates and/or point locations.
-#' @param ... Unused.
-#' @return An A-matrix.
-#' @author Fabian E. Bachl \email{bachlfab@@gmail.com}
-#' @rdname amatrix_eval
+#' @param input Component inputs, from `input_eval()`
+#' @param linearisation evaluation state
+#' @param ... Optional parameters passed on to `ibm_eval`
+#' and `ibm_jacobian.
+#' @return A `bru_mapper_taylor` or `comp_simple_list` object.
+#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
+#' @rdname comp_lin_eval
 
-amatrix_eval.component <- function(component, data, ...) {
-  val <- input_eval(component, data)
-  A <- ibm_amatrix(component$mapper, input = val, ...)
-
-  if (!is.null(val[["weights"]])) {
-    A <- val[["weights"]] * A
+comp_lin_eval.component <- function(component,
+                                    input = NULL,
+                                    state = NULL,
+                                    ...) {
+  if (is.null(state)) {
+    state <- rep(0, ibm_n(component[["mapper"]]))
   }
-
-  # Mask columns of A
-  # TODO: check what this feature is intended for!
-  if (!is.null(component$A.msk)) {
-    A[, as.logical(component$A.msk)] <- 0.0
-  }
-
-  A
+  ibm_linear(component[["mapper"]], input = input, state = state, ...)
 }
 
 #' @export
-#' @rdname amatrix_eval
+#' @rdname comp_lin_eval
 
-amatrix_eval.component_list <- function(components, data, ...) {
-  lapply(components, function(x) amatrix_eval(x, data = data, ...))
+comp_lin_eval.component_list <- function(components, input, state, ...) {
+  # Note: Make sure the list element names carry over!
+  mappers <-
+    lapply(components,
+           function(x) {
+             label <- x[["label"]]
+             comp_lin_eval(x,
+                           input = input[[label]],
+                           state = state[[label]],
+                           ...)
+           })
+
+  class(mappers) <- c("comp_simple_list", class(mappers))
+  mappers
 }
 
 #' @section Simple covariates and the map parameter:
@@ -1479,29 +1575,31 @@ amatrix_eval.component_list <- function(components, data, ...) {
 #'
 #' \itemize{\item{`formula = y ~ f(xsquared, model = "linear")`,}}
 #'
-#' In inlabru this can be achived using two ways of using the `main` parameter
-#' (`map` in version 2.1.13 and earlier).
+#' In inlabru this can be achieved in several ways of using the `main` parameter
+#' (`map` in version 2.1.13 and earlier), which does not need to be named.
 #'
 #' \itemize{
 #' \item{`components = y ~ psi(main = x^2, model = "linear")`}
-#' \item{`components = y ~ psi(main = mySquareFun(x), model = "linear")`,}
-#' \item{`components = y ~ psi(main = myOtherSquareFun, model = "linear")`,}
+#' \item{`components = y ~ psi(x^2, model = "linear")`}
+#' \item{`components = y ~ psi(mySquareFun(x), model = "linear")`,}
+#' \item{`components = y ~ psi(myOtherSquareFun, model = "linear")`,}
 #'
 #' }
 #'
 #' In the first example inlabru will interpret the map parameter as an expression to be evaluated within
 #' the data provided. Since \eqn{x} is a known covariate it will know how to calculate it. The second
-#' example is an expression as well but it uses a function alled `mySquareFun`. This function is
-#' defined by user but has wo be accessible within the work space when setting up the compoonents.
-#' The third example provides the function `myOtherSquareFun` directly and not within an expression.
-#' In this case, inlabru will call the function using the data provided via the  `data` parameter.
-#' inlabru expects that the output of this function is a data.frame with "psi" being the name of the
-#' single existing column. For instance,
-#'
-#' \code{myOtherSquareFun = function(data) {
-#'                             data = data[,"x", drop = FALSE] ;
-#'                             colnames(data) = "psi" ;
-#'                             return(data)}}
+#' example is an expression as well but it uses a function called `mySquareFun`. This function is
+#' defined by user but has to be accessible within the work space when setting up the components.
+#' The third example provides the function `myOtherSquareFun`. In this case,
+#'  inlabru will call the function as `myOtherSquareFun(.data.)`, where `.data.`
+#'  is the data provided via the [like()] `data` parameter.
+#' The function needs to know what parts of the data to use to construct the
+#' needed output. For example,
+#' ```
+#' myOtherSquareFun <- function(data) {
+#'   data[ ,"x"]^2
+#' }
+#' ```
 #'
 #' @section Spatial Covariates:
 #'
@@ -1510,39 +1608,47 @@ amatrix_eval.component_list <- function(components, data, ...) {
 #' data frame or write a covariate function like in the previous section there is an even more
 #' convenient way in inlabru. Spatial covariates are often stored as `SpatialPixelsDataFrame`,
 #' `SpatialPixelsDataFrame` or `RasterLayer` objects. These can be provided directly via
-#' the map parameter if the input data is a `SpatialPointsDataFrame`. inlabru will automatically
-#' evaluate and/or interpolate the coariate at your data locations when using code like
-#'
-#' \itemize{\item{`components = y ~ psi(mySpatialPixels, model = "linear")`.}}
+#' the input expressions if the [like()] data is a `SpatialPointsDataFrame` object.
+#' inlabru will automatically
+#' evaluate and/or interpolate the covariate at your data locations when using code like
+#' ```
+#' components = y ~ psi(mySpatialPixels, model = "linear")
+#' ```
 #'
 #' @section Coordinates:
 #'
 #' A common spatial modelling component when using inla are SPDE models. An important feature of
-#' inlabru is that it will automatically calculate the so called A-matrix which maps SPDE
-#' values at the mesh vertices to values at the data locations. For this purpose, the map parameter
-#' can be se to `coordinates`, which is the `sp` package function that extracts point
-#' coordinates from the SpatialPointsDataFrame that was provided as input to bru. The code for
+#' inlabru is that it will automatically calculate the so called A-matrix (a component model matrix)
+#' which maps SPDE
+#' values at the mesh vertices to values at the data locations. For this purpose, the input
+#' can be set to `coordinates`, which is the `sp` package function that extracts point
+#' coordinates from the `SpatialPointsDataFrame` that was provided as input to [like()]. The code for
 #' this would look as follows:
-#'
-#' \itemize{\item{`components = y ~ mySPDE(main = coordinates, model = inla.spde2.matern(...))`.}}
+#' ```
+#' components = y ~ mySPDE(main = coordinates, model = inla.spde2.matern(...))
+#' ```
 #'
 #' @export
 #' @keywords internal
 #' @param component A component.
-#' @param data A `data.frame` or Spatial* object of covariates and/or point locations. If null, return the component's map.
+#' @param data A `data.frame` or `Spatial*` object of covariates and/or point locations.
+#' If `NULL`, return the component's map.
 #' @param ... Unused.
-#' @return An vector or a coordinate matrix
-#' @author Fabian E. Bachl \email{bachlfab@@gmail.com}
+#' @return An list of mapper input values, formatted for the full component mapper
+#' (of type `bru_mapper_scale`)
+#' @author Fabian E. Bachl \email{bachlfab@@gmail.com}, Finn Lindgren \email{finn.lindgren@@gmail.com}
 #' @rdname input_eval
 
 input_eval.component <- function(component,
                                  data,
                                  ...) {
-  val <- list()
+  stopifnot(inherits(component[["mapper"]], "bru_mapper_scale"))
+
   # The names should be a subset of main, group, replicate
-  part_names <- names(component[["mapper"]])
+  part_names <- names(component[["mapper"]][["mapper"]])
+  mapper_val <- list()
   for (part in part_names) {
-    val[[part]] <-
+    mapper_val[[part]] <-
       input_eval(
         component[[part]]$input,
         data,
@@ -1551,8 +1657,10 @@ input_eval.component <- function(component,
         ...
       )
   }
-  if (!is.null(component[["weights"]])) {
-    val[["weights"]] <-
+  if (is.null(component[["weights"]])) {
+    scale_val <- NULL
+  } else {
+    scale_val <-
       input_eval(
         component[["weights"]],
         data,
@@ -1563,7 +1671,7 @@ input_eval.component <- function(component,
   }
   # Any potential length mismatches must be handled by bru_mapper_multi and
   # bru_mapper_collect, since e.g. 'main' might take a list() input object.
-  val
+  list(mapper = mapper_val, scale = scale_val)
 }
 
 #' @export
@@ -1573,17 +1681,42 @@ input_eval.component_list <-
   function(components,
            data,
            ...) {
-    part <- match.arg(part)
-    result <- lapply(components, function(x) input_eval(x, data = data, ...))
-    result
+    lapply(components, function(x) input_eval(x, data = data, ...))
   }
 
 
 
+input_eval_layer <- function(layer, selector = NULL, envir, enclos,
+                             label,
+                             e_input) {
+  input_layer <- tryCatch(
+    eval(layer, envir = envir, enclos = enclos),
+    error = function(e) {
+      e
+    }
+  )
+  if (inherits(input_layer, "error")) {
+    stop(paste0(
+      "Failed to evaluate 'layer' input '",
+      deparse(layer),
+      "' for '",
+      paste0(label, ":layer"),
+      "'."
+    ))
+  }
+  if (is.null(input_layer) && is.null(selector)) {
+    if (label %in% names(e_input)) {
+      input_layer <- label
+    }
+  }
+  input_layer
+}
+
+
 #' @export
 #' @rdname input_eval
 
-input_eval.bru_input <- function(input, data, env = NULL, label = NULL,
+input_eval.bru_input <- function(input, data, env = NULL,
                                  null.on.fail = FALSE, ...) {
 
   # Evaluate the map with the data in an environment
@@ -1606,7 +1739,7 @@ input_eval.bru_input <- function(input, data, env = NULL, label = NULL,
   }
   assign(".data.", data, envir = envir)
 
-  emap <- tryCatch(eval(input$input, envir = envir, enclos = enclos),
+  e_input <- tryCatch(eval(input$input, envir = envir, enclos = enclos),
     error = function(e) {
     }
   )
@@ -1627,15 +1760,15 @@ input_eval.bru_input <- function(input, data, env = NULL, label = NULL,
   # ## A matrix.
   #  n <- nrow(as.data.frame(data))
 
-  if (is.null(emap)) {
+  if (is.null(e_input)) {
     if (null.on.fail) {
       return(NULL)
     }
     #    val <- rep(1, n)
     val <- 1
-  } else if (is.function(emap)) {
+  } else if (is.function(e_input)) {
     # Allow but detect failures:
-    val <- tryCatch(emap(data),
+    val <- tryCatch(e_input(data),
       error = function(e) {
       }
     )
@@ -1665,9 +1798,9 @@ input_eval.bru_input <- function(input, data, env = NULL, label = NULL,
         }
       )
     }
-  } else if (inherits(emap, "formula")) {
+  } else if (inherits(e_input, "formula")) {
     # Allow but detect failures:
-    val <- tryCatch(MatrixModels::model.Matrix(emap, data = data, sparse = TRUE),
+    val <- tryCatch(MatrixModels::model.Matrix(e_input, data = data, sparse = TRUE),
       error = function(e) {
       }
     )
@@ -1677,28 +1810,41 @@ input_eval.bru_input <- function(input, data, env = NULL, label = NULL,
       # that are likely to happen for multilikelihood models; A component only
       # needs to be evaluable for at least one of the likelihoods.
     }
-  } else if (inherits(emap, "SpatialGridDataFrame") |
-    inherits(emap, "SpatialPixelsDataFrame")) {
-    if (is.null(input[["selector"]])) {
-      layer <-
-        if (is.null(input[["layer"]])) {
-          1
-        } else {
-          input[["layer"]]
-        }
-    }
-    val <- eval_SpatialDF(
-      emap,
+  } else if (inherits(
+    e_input,
+    c(
+      "SpatialGridDataFrame",
+      "SpatialPixelsDataFrame",
+      "SpatRaster"
+    )
+  )) {
+    input_layer <-
+      input_eval_layer(
+        layer = input[["layer"]],
+        selector = input[["selector"]],
+        envir = envir,
+        enclos = enclos,
+        label = input[["label"]],
+        e_input = e_input
+      )
+    layer <- extract_layer(
+      data,
+      input_layer,
+      input[["selector"]]
+    )
+    check_layer(e_input, data, layer)
+    val <- eval_spatial(
+      e_input,
       data,
       layer = layer,
-      selector = input[["selector"]]
+      selector = NULL
     )
-    #  } else if ((input$label == "offset") &&
-    #    is.numeric(emap) &&
-    #    (length(emap) == 1)) {
-    #    val <- rep(emap, n)
+    #  } else if ((input$label %in% c("offset", "const")) &&
+    #    is.numeric(e_input) &&
+    #    (length(e_input) == 1)) {
+    #    val <- rep(e_input, n)
   } else {
-    val <- emap
+    val <- e_input
   }
 
   # ## Need to allow different sizes; K %*% effect needs to have same length as response, but the input and effect effect itself doesn't.
@@ -1711,14 +1857,18 @@ input_eval.bru_input <- function(input, data, env = NULL, label = NULL,
   # to fix that by filling in nearest neighbour values.
   # # TODO: Check how to deal with this fully in the case of multilikelihood models
   # Answer: should respect the lhood "include/exclude" info for the component list
-  if ((inherits(emap, "SpatialGridDataFrame") ||
-    inherits(emap, "SpatialPixelsDataFrame")) &&
+  if ((inherits(e_input, c(
+    "SpatialGridDataFrame",
+    "SpatialPixelsDataFrame",
+    "SpatRaster"
+  ))) &&
     any(is.na(as.data.frame(val)))) {
     warning(
       paste0(
         "Model input '",
         deparse(input$input),
-        "' for '", label, "' returned some NA values.\n",
+        "' for '", input$label,
+        "' returned some NA values.\n",
         "Attempting to fill in spatially by nearest available value.\n",
         "To avoid this basic covariate imputation, supply complete data."
       ),
@@ -1726,8 +1876,8 @@ input_eval.bru_input <- function(input, data, env = NULL, label = NULL,
     )
 
     val <- bru_fill_missing(
-      data = emap, where = data, values = val,
-      layer = layer, selector = input[["selector"]]
+      data = e_input, where = data, values = val,
+      layer = layer, selector = NULL
     )
   }
 
@@ -1764,7 +1914,7 @@ input_eval.bru_input <- function(input, data, env = NULL, label = NULL,
 #' @rdname index_eval
 
 index_eval.component <- function(component, inla_f, ...) {
-  idx <- ibm_values(component[["mapper"]], inla_f = inla_f, multi = 1)
+  idx <- ibm_values(component[["mapper"]], inla_f = inla_f, multi = TRUE)
   names(idx) <- paste0(component[["label"]], c("", ".group", ".repl"))
   idx
 }
@@ -1785,5 +1935,5 @@ index_eval.component_list <- function(components, inla_f, ...) {
 #' @rdname inla_subset_eval
 
 inla_subset_eval.component_list <- function(components, ...) {
-  lapply(components, function(x) ibm_inla_subset(x[["mapper"]], multi = 1))
+  lapply(components, function(x) ibm_inla_subset(x[["mapper"]], multi = TRUE))
 }
