@@ -1,12 +1,10 @@
-
-
 #' Create an inlabru model object from model components
 #'
 #' The [inlabru] syntax for model formulae is different from what
 #' `INLA::inla` considers a valid.
 #' In inla most of the effects are defined by adding an `f(...)` expression to the formula.
-#' In [inlabru] the `f` is replaced by an arbitrary (exception: `offset`) string that will
-#' determine the label of the effect. See Details for further information.
+#' In [inlabru] the `f` is replaced by an arbitrary (exceptions: `const` and `offset`)
+#' string that will determine the label of the effect. See Details for further information.
 #'
 #' @details
 #' For instance
@@ -86,7 +84,7 @@ bru_model <- function(components, lhoods) {
 
   for (cmp in included) {
     if (linear ||
-      !identical(components[[cmp]][["main"]][["type"]], "offset")) {
+      !(components[[cmp]][["main"]][["type"]] %in% c("offset", "const"))) {
       formula <- update.formula(formula, components[[cmp]]$inla.formula)
     }
   }
@@ -108,10 +106,11 @@ bru_model <- function(components, lhoods) {
 #'
 #' @export
 #' @param model A [bru] model
-#' @param state list of lists, as generated by [evaluate_state()]
+#' @param state list of state lists, as generated by [evaluate_state()]
 #' @param data A `list`, `data.frame`, or `Spatial*DataFrame`, with coordinates
 #' and covariates needed to evaluate the predictor.
-#' @param A Precomputed A-matrices
+#' @param input Precomputed inputs list for the components
+#' @param comp_simple Precomputed `comp_simple_list` for the components
 #' @param predictor A formula or an expression to be evaluated given the
 #' posterior or for each sample thereof. The default (`NULL`) returns a
 #' `data.frame` containing the sampled effects. In case of a formula the right
@@ -145,7 +144,8 @@ bru_model <- function(components, lhoods) {
 evaluate_model <- function(model,
                            state,
                            data = NULL,
-                           A = NULL,
+                           input = NULL,
+                           comp_simple = NULL,
                            predictor = NULL,
                            format = NULL,
                            include = NULL,
@@ -156,17 +156,23 @@ evaluate_model <- function(model,
   if (is.null(state)) {
     stop("Not enough information to evaluate model states.")
   }
-  if (is.null(A) && !is.null(data)) {
-    A <- amatrix_eval(model$effects[included], data = data, inla_f = TRUE)
+  if (is.null(input) && !is.null(data)) {
+    input <- input_eval(components = model$effects[included],
+                        data = data,
+                        inla_f = TRUE)
   }
-  if (is.null(A)) {
+  if (is.null(comp_simple) && !is.null(input)) {
+    comp_simple <- evaluate_comp_simple(model$effects[included],
+                                        input = input,
+                                        inla_f = TRUE)
+  }
+  if (is.null(comp_simple)) {
     effects <- NULL
   } else {
-    effects <- evaluate_effect_multi(
-      model$effects[included],
+    effects <- evaluate_effect_multi_state(
+      comp_simple,
       state = state,
-      data = data,
-      A = A
+      input = input
     )
   }
 
@@ -237,13 +243,13 @@ evaluate_state <- function(model,
 
 #' @export
 #' @rdname evaluate_effect
-evaluate_effect_single <- function(...) {
-  UseMethod("evaluate_effect_single")
+evaluate_effect_single_state <- function(...) {
+  UseMethod("evaluate_effect_single_state")
 }
 #' @export
 #' @rdname evaluate_effect
-evaluate_effect_multi <- function(...) {
-  UseMethod("evaluate_effect_multi")
+evaluate_effect_multi_state <- function(...) {
+  UseMethod("evaluate_effect_multi_state")
 }
 
 #' Evaluate a component effect
@@ -253,110 +259,78 @@ evaluate_effect_multi <- function(...) {
 #'
 #' @export
 #' @keywords internal
-#' @param component A `bru_component`.
-#' @param data A `data.frame` or Spatial* object of covariates and/or point locations.
-#' @param state Specification of one (for `evaluate_effect_single`) or several
-#' (for `evaluate_effect_multi`) latent variable states:
-#' * `evaluate_effect_single.component`: A vector of the latent component state.
-#' * `evaluate_effect_single.component_list`: list of named vectors.
-#' * `evaluate_effect_multi`: list of lists of the `evaluate_effect_single.component_list` type.
-#' For `evaluate_effect_multi.component`,
-#' only the label for the given component needs to be included
-#' @param A A matrix overriding the default projection matrix or matrices
-#' (named list of matrices for `evaluate_effect.component_list`)
-#' @param ... Unused.
-#' @return * `evaluate_effect_single.component`: A numeric vector of the component effect values
-#' state.
+#' @param component A `bru_component`, `comp_simple`, or `comp_simple_list`.
+#' @param input Pre-evaluated component input
+#' @param state Specification of one (for `evaluate_effect_single_state`) or several
+#' (for `evaluate_effect_multi_State`) latent variable states:
+#' * `evaluate_effect_single_state.bru_mapper`: A vector of the latent component state.
+#' * `evaluate_effect_single_state.*_list`: list of named state vectors.
+#' * `evaluate_effect_multi_state.*_list`: list of lists of named state vectors.
+#' @param ... Optional additional parameters, e.g. `inla_f`. Normally unused.
 #' @author Fabian E. Bachl \email{bachlfab@@gmail.com} and
 #' Finn Lindgren \email{finn.lindgren@@gmail.com}
 #' @rdname evaluate_effect
 
-evaluate_effect_single.component <- function(component, state, data, A = NULL, ...) {
-  # Make A-matrix (if not provided)
-  if (is.null(A)) {
-    A <- amatrix_eval(component, data)
-  }
-
-  # Determine component depending on the type of latent model
-  if (component$main$type %in% c("offset")) {
-    values <- A
-  } else {
-    values <- A %*% state
-  }
+evaluate_effect_single_state.bru_mapper <- function(component, input, state,
+                                                     ...) {
+  values <- ibm_eval(component, input = input, state = state, ...)
 
   as.vector(as.matrix(values))
 }
 
-
-#' @return * `evaluate_effect_single.component_list`: A list of evaluated
+#' @return * `evaluate_effect_single_state.component_list`: A list of evaluated
 #' component effect values
 #' @export
 #' @rdname evaluate_effect
 #' @keywords internal
-evaluate_effect_single.component_list <- function(components, state, data,
-                                                  A = NULL, ...) {
-  if (is.null(A)) {
-    A <- amatrix_eval(components, data = data)
-  }
+evaluate_effect_single_state.comp_simple_list <- function(components,
+                                                          input,
+                                                          state,
+                                                          ...) {
   result <- list()
   for (label in names(components)) {
-    result[[label]] <- evaluate_effect_single(
+    result[[label]] <- evaluate_effect_single_state(
       components[[label]],
-      data = data,
+      input = input[[label]],
       state = state[[label]],
-      A = A[[label]]
+      ...
     )
   }
   result
 }
 
-#' @return * `evaluate_effect_multi.component`: A list of numeric vectors
-#' of evaluated component effects.
+#' @return * `evaluate_effect_multi.comp_simple_list`: A list of lists of
+#' evaluated component effects, one list for each state
 #' @export
 #' @rdname evaluate_effect
 #' @keywords internal
-evaluate_effect_multi.component <- function(component, state, data,
-                                            A = NULL, ...) {
-  if (is.null(A)) {
-    A <- amatrix_eval(component, data = data)
-  }
+evaluate_effect_multi_state.comp_simple_list <- function(components, input, state, ...) {
   lapply(
     state,
     function(x) {
-      evaluate_effect_single(
-        component,
+      evaluate_effect_single_state(
+        components,
+        input = input,
         state = x,
-        data = data,
-        A = A,
         ...
       )
     }
   )
 }
 
-
-#' @return * `evaluate_effect_multi.component_list`: A list of lists of
-#' evaluated component effects, one list for each state
 #' @export
 #' @rdname evaluate_effect
 #' @keywords internal
-evaluate_effect_multi.component_list <- function(components, state, data,
-                                                 A = NULL, ...) {
-  if (is.null(A)) {
-    A <- amatrix_eval(components, data = data)
-  }
-  lapply(
-    state,
-    function(x) {
-      evaluate_effect_single(
-        components,
-        state = x,
-        data = data,
-        A = A,
-        ...
-      )
-    }
-  )
+evaluate_effect_single_state.component_list <- function(components, input, state, ...) {
+  comp_simple <- evaluate_comp_simple(components, input = input, ...)
+  evaluate_effect_single_state(comp_simple, input = input, state = state, ...)
+}
+#' @export
+#' @rdname evaluate_effect
+#' @keywords internal
+evaluate_effect_multi_state.component_list <- function(components, input, state, ...) {
+  comp_simple <- evaluate_comp_simple(components, input = input, ...)
+  evaluate_effect_multi_state(comp_simple, input = input, state = state, ...)
 }
 
 
@@ -372,7 +346,7 @@ evaluate_effect_multi.component_list <- function(components, state, data,
 #' @param state A list where each element is a list of named latent state
 #' information, as produced by [evaluate_state()]
 #' @param effects A list where each element is list of named evaluated effects,
-#' as computed by [evaluate_effect_multi.component_list()]
+#' as computed by [evaluate_effect_multi_state.component_list()]
 #' @param predictor Either a formula or expression
 #' @param format character; determines the storage format of the output.
 #' Available options:
@@ -451,7 +425,7 @@ evaluate_predictor <- function(model,
 
   eval_fun_factory <-
     function(.comp, .envir, .enclos) {
-      .is_offset <- .comp$main$type %in% c("offset")
+      .is_offset <- .comp$main$type %in% c("offset", "const")
       .is_iid <- .comp$main$type %in% c("iid")
       .mapper <- .comp$mapper
       .label <- paste0(.comp$label, "_latent")
@@ -460,6 +434,7 @@ evaluate_predictor <- function(model,
       .iid_cache_index <- NULL
       eval_fun <- function(main, group = NULL,
                            replicate = NULL,
+                           weights = NULL,
                            .state = NULL) {
         if (is.null(group)) {
           group <- rep(1, NROW(main))
@@ -467,59 +442,59 @@ evaluate_predictor <- function(model,
         if (is.null(replicate)) {
           replicate <- rep(1, NROW(main))
         }
-        .A <- ibm_amatrix(
-          .mapper,
-          input = list(
+        if (!.is_offset && is.null(.state)) {
+          .state <- eval(
+            parse(text = .label),
+            envir = .envir,
+            enclos = .enclos
+          )
+        }
+        .input = list(
+          mapper = list(
             main = main,
             group = group,
             replicate = replicate
-          )
+          ),
+          scale = weights
         )
-        if (.is_offset) {
-          .values <- .A
-        } else {
-          if (is.null(.state)) {
-            .state <- eval(parse(text = .label),
+
+        .values <- ibm_eval(
+          .mapper,
+          input = .input,
+          state = .state
+        )
+        if (.is_iid) {
+          ok <- ibm_valid_input(
+            .mapper,
+            input = .input
+          )
+          if (any(!ok)) {
+            .cache_state_index <- eval(
+              parse(text = ".cache_state_index"),
               envir = .envir,
               enclos = .enclos
             )
-          }
-          .values <- .A %*% .state
-          if (.is_iid) {
-            ok <- ibm_valid_input(
-              .mapper,
-              input = list(
-                main = main,
-                group = group,
-                replicate = replicate
-              )
-            )
-            if (any(!ok)) {
-              .cache_state_index <- eval(parse(text = ".cache_state_index"),
+            if (!identical(.cache_state_index, .iid_cache_index)) {
+              .iid_cache_index <<- .cache_state_index
+              .iid_cache <<- list()
+            }
+            key <- as.character(main[!ok])
+            not_cached <- !(key %in% names(.iid_cache))
+            if (any(not_cached)) {
+              .prec <- eval(
+                parse(text = .iid_precision),
                 envir = .envir,
                 enclos = .enclos
               )
-              if (!identical(.cache_state_index, .iid_cache_index)) {
-                .iid_cache_index <<- .cache_state_index
-                .iid_cache <<- list()
+              for (k in unique(key[not_cached])) {
+                .iid_cache[k] <<- rnorm(1, mean = 0, sd = .prec^-0.5)
               }
-              key <- as.character(main[!ok])
-              not_cached <- !(key %in% names(.iid_cache))
-              if (any(not_cached)) {
-                .prec <- eval(parse(text = .iid_precision),
-                  envir = .envir,
-                  enclos = .enclos
-                )
-                for (k in unique(key[not_cached])) {
-                  .iid_cache[k] <<- rnorm(1, mean = 0, sd = .prec^-0.5)
-                }
-              }
-              .values[!ok] <- vapply(
-                key,
-                function(k) .iid_cache[[k]],
-                0.0
-              )
             }
+            .values[!ok] <- vapply(
+              key,
+              function(k) .iid_cache[[k]],
+              0.0
+            )
           }
         }
 
@@ -620,23 +595,156 @@ component_eval <- function(main,
                            replicate = NULL,
                            .state = NULL) {
   stop(paste0(
-    "In your predictor expression, use the literal name of a component instead\n",
-    "of 'component_eval(...)', not eval_fun.  See ?component_eval for more information."
+    "In your predictor expression, use 'mylabel_eval(...)' instead of\n",
+    "'component_eval(...)'.  See ?component_eval for more information."
   ))
 }
 
 
 
 
-#' Compute all A-matrices
+
+#' Compute all component linearisations
 #'
-#' Computes the A matrices for included components for each model likelihood
+#' Computes individual `bru_mapper_taylor` objects for included components
+#' for each model likelihood
 #'
 #' @param model A `bru_model` object
-#' @param lhoods A `bru__like_list` object
+#' @param input A list of named lists of component inputs
+#' @param state A named list of component states
+#' @param inla_f Controls the input data interpretations
+#' @return A list (class 'comp_simple') of named lists (class 'comp_simple_list')
+#' of `bru_mapper_taylor` objects,
+#' one for each included component
+#' @rdname evaluate_comp_lin
+evaluate_comp_lin <- function(model, input, state, inla_f = FALSE) {
+  stopifnot(inherits(model, "bru_model"))
+  mappers <-
+    lapply(
+      input,
+      function(inp) {
+        included <- parse_inclusion(
+          names(model[["effects"]]),
+          names(inp),
+          NULL
+        )
+
+        mappers <- comp_lin_eval(
+          model[["effects"]][included],
+          input = inp[included],
+          state = state[included],
+          inla_f = inla_f
+        )
+
+        class(mappers) <- c("comp_simple_list", class(mappers))
+        mappers
+      }
+    )
+
+  class(mappers) <- c("comp_simple_list_list", class(mappers))
+  mappers
+}
+
+#' Compute simplified component mappings
+#'
+#' Computes individual `bru_mapper_taylor` objects for included linear components
+#' for each model likelihood, and keeps non-linear mappers intact.
+#'
+#' @param model A `bru_model` object
+#' @param input A list of named lists of component inputs
+#' @param inla_f Controls the input data interpretations
+#' @return A list (class 'comp_simple_list_list') of named lists (class 'comp_simple_list')
+#' of `bru_mapper` objects,
+#' one for each included component
+#' @export
+#' @keywords internal
+#' @rdname evaluate_comp_simple
+evaluate_comp_simple <- function(...) {
+  UseMethod("evaluate_comp_simple")
+}
+
+#' @export
+#' @rdname evaluate_comp_simple
+evaluate_comp_simple.component_list <- function(components, input,
+                                                inla_f = FALSE, ...) {
+  are_linear <- vapply(components,
+                       function(x) ibm_is_linear(x[["mapper"]]),
+                       TRUE)
+  the_linear <- names(components)[are_linear]
+  the_nonlinear <- names(components)[!are_linear]
+
+  if (any(are_linear)) {
+    mappers <- comp_lin_eval(
+      components[the_linear],
+      input = input[the_linear],
+      state = NULL,
+      inla_f = inla_f
+    )
+  } else {
+    mappers <- list()
+  }
+
+  if (any(!are_linear)) {
+    warning("Non-linear mappers are experimental!", immediate. = TRUE)
+    mappers <- c(mappers,
+                 lapply(components[the_nonlinear],
+                        function(x) x[["mapper"]]))
+  }
+
+  # Reorder
+  mappers <- mappers[names(components)]
+
+  class(mappers) <- c("comp_simple_list", class(mappers))
+  mappers
+}
+
+#' @export
+#' @rdname evaluate_comp_simple
+evaluate_comp_simple.bru_model <- function(model, input, ...) {
+  mappers <-
+    lapply(
+      input,
+      function(inp) {
+        included <- parse_inclusion(
+          names(model[["effects"]]),
+          names(inp),
+          NULL
+        )
+
+        evaluate_comp_simple(model[["effects"]][included], input = inp, ...)
+      }
+    )
+
+  class(mappers) <- c("comp_simple_list_list", class(mappers))
+  mappers
+}
+
+#' Subsetting of comp_simple_list objects, retaining class
+#' @export
+#' @param x `comp_simple_list` object from which to extract element(s)
+#' @param i indices specifying elements to extract
+#' @keywords internal
+#' @rdname evaluate_comp_simple_list_subsetting
+`[.comp_simple_list` <- function(x, i) {
+  env <- environment(x)
+  object <- NextMethod()
+  class(object) <- c("comp_simple_list", "list")
+  environment(object) <- env
+  object
+}
+
+
+
+#' Compute all component inputs
+#'
+#' Computes the component inputs for included components
+#' for each model likelihood
+#'
+#' @param model A `bru_model` object
+#' @param lhoods A `bru_like_list` object
 #' @param inla_f logical
-#' @rdname evaluate_A
-evaluate_A <- function(model, lhoods, inla_f) {
+#' @rdname evaluate_inputs
+evaluate_inputs <- function(model, lhoods, inla_f) {
   stopifnot(inherits(model, "bru_model"))
   lapply(
     lhoods,
@@ -647,7 +755,8 @@ evaluate_A <- function(model, lhoods, inla_f) {
         lh[["exclude_components"]]
       )
 
-      amatrix_eval(model$effects[included],
+      input_eval(
+        model$effects[included],
         data = lh[["data"]],
         inla_f = inla_f
       )
